from Common import *


class Transaction(Object):
	'''
	A transaction is a container for the transactionual information of a transaction. This includes the application, request, response and session. It also includes the initial servlet for which the request was made and the current servlet executing in the case of nested servlets.
	'''

	## Init ##
	
	def __init__(self, application, request=None):
		Object.__init__(self)
		self._application   = application
		self._request       = request
		self._response      = None
		self._session       = None
		self._servlet       = None
		self._errorOccurred = 0
	

	## Access ##
	
	def application(self):
		return self._application

	def request(self):
		return self._request
	
	def response(self):
		return self._response
		
	def setResponse(self, response):
		self._response = response

	def session(self):
		return self._session
	
	def setSession(self, session):
		self._session = session

	def servlet(self):
		''' Return the current servlet that is processing. Remember that servlets can be nested. '''
		return self._servlet

	def setServlet(self, servlet):
		self._servlet = servlet

	def duration(self):
		''' Returns the duration, in seconds, of the transaction (basically response end time minus request start time). '''
		return self._response.endTime() - self._request.time()

	def errorOccurred(self):
		return self._errorOccurred

	def setErrorOccurred(self, flag):
		''' Invoked by the application if an exception is raised to the application level. '''
		self._errorOccurred = flag

		
	## Debugging ##
	
	def dump(self, f=sys.stdout):
		''' Dumps debugging info to stdout. '''
		f.write('>> Transaction: %s\n' % self)
		for attr in dir(self):
			f.write('%s: %s\n' % (attr, getattr(self, attr)))
		f.write('\n')
